// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: PixelIterator
// Author: Hassan Siddiqui
//
// Description: 
//
// Allows one to iterate across a structure of pixels.
// Useful for traversing detector maps
// as described in email XMM-SOC/HS/1999-100,
// or detector masks.
//
// Modifications: See ChangeLog
// 
// $Id: PixelIterator.h,v 1.1 2012/01/25 15:46:31 rsaxton Exp $

#ifndef PIXELITERATOR_H
#define PIXELITERATOR_H

// General classes

// SAS-specific classes

// General utility functions

// Local classes
#include "GeneralIterator.h"
#include "PixelElement.h"
#include "WCSConvertor.h"

class PixelIterator : public GeneralIterator<PixelElement>
{
public:

  /* Constructor */
  PixelIterator();

  /* Destructor */
  virtual ~PixelIterator();

  /* The total number of elements in map */
  virtual unsigned size() const = 0;
  virtual unsigned xbins() const = 0;

  /* The bounds of map in physical coordinates */
  /* [HS] new methods as a result of new WCSConvertor i/f -
    X and Y positions now treated as a related pair */
  virtual CoordPair physXYmin() const  ;
  virtual CoordPair physXYmax() const  ;

  /* WCSConvertor object */
  virtual const WCSConvertor * wcsconvertor() const = 0;

  /* Get the normalised pixel value */
  PixelElement normalisedItem();

  void normalise();

  bool hasValidNormalisation();

private:

  double _isNormalised;
  double _normalisation;
  bool   _hasValidNormalisation;
};

#endif
