// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: testCopyArray
// Author: Hassan Siddiqui
//
// Description: Test harness for CopyUtils class
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: testCopyArray.cc,v 1.1 2012/01/25 15:46:31 rsaxton Exp $




// General classes

#include <iostream>


// SAS-specific classes

#include "param.h"
#include "Dal.h"

// General utility functions
#include "CopyUtils.h"
#include "Tracer.h"

static const unsigned long NUMBER_OF_ELEMENTS_X = 10;
static const unsigned long NUMBER_OF_ELEMENTS_Y = 20;
 
// ---------------------------------------------------

// Main function

// ---------------------------------------------------

int main()
{
  PROC;

  // Write dataset

  DataSet* fromSet = dataSetServer->open("testCopyArray1.fits", DataSet::Create);

  vector<unsigned long> size;
  size.push_back(NUMBER_OF_ELEMENTS_Y);
  size.push_back(NUMBER_OF_ELEMENTS_X);

  Array* fromArray = fromSet->addArray("FROM_Array", DataSet::Real32, size);

  // Add a few attributes

  *fromArray->addAttribute("ATT1",    "This is a test attribute")  = "VALUE1";

  *fromArray->addAttribute("ATT2",    "This is a test attribute")  = "VALUE2";

  // Populate the data

  for (unsigned long i  = 0; i < fromArray->data()->elements(); ++i)
    {
      (fromArray->data()->real32Data())[i] = double(i);
    }

  
  // Now create a new dataset

  
  DataSet* toSet = dataSetServer->open("testCopyArray2.fits", DataSet::Create);


  // Copy the array from the first dataset to the new one

  const Array* newarray = CopyUtils::copyArrayToDataSet(fromArray, toSet);

  const Array* copy_newarray = CopyUtils::copyArrayToDataSet(fromArray, toSet, "A_COPY");

  // Do some checks

  assert(fromArray->data()->elements() == newarray->data()->elements());
  assert(fromArray->data()->elements() == copy_newarray->data()->elements());

  dataSetServer->close(toSet);

  dataSetServer->close(fromSet);


  RETURN 0;
} 
