// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// This  skeleton file was automatically created by 
//            pkgmaker 0.1
//            Thursday, December 11 1997
// 
// Module: testCopyTable
// Author: Hassan Siddiqui
//
// Description: Test harness for CopyUtils class
// 
// Comments: 
//  
// Modifications: See ChangeLog
// 
// $Id: testCopyTable.cc,v 1.1 2012/01/25 15:46:31 rsaxton Exp $




// General classes

#include <iostream>


// SAS-specific classes

#include "param.h"
#include "Dal.h"

// General utility functions
#include "CopyUtils.h"
#include "Tracer.h"

static const unsigned long NUMBER_OF_ROWS = 10;
 
// ---------------------------------------------------

// Main function

// ---------------------------------------------------

int main()
{
  PROC;

  // Write dataset

  DataSet* fromSet = dataSetServer->open("testCopyTable1.fits", DataSet::Create);

  Table* fromTable = fromSet->addTable("FROM_TABLE", NUMBER_OF_ROWS);

  // Add a few attributes

  *fromTable->addAttribute("ATT1",    "This is a test attribute")  = "VALUE1";

  *fromTable->addAttribute("ATT2",    "This is a test attribute")  = "VALUE2";

  // Add a few columns

  // Int16 column

  string namei16 = "TESTI16_COLUMN";

  Column* testi16Col = fromTable->addColumn(namei16, Column::Int16, "int16 test data", "");
  ColumnData* testi16ColData = testi16Col->data();
  int16* testi16Data = testi16ColData->int16Data();
  

  // Real32 column

  string namer32 = "TESTR32_COLUMN";

  Column* testr32Col = fromTable->addColumn(namer32, Column::Real32, "real32 test data", "");
  ColumnData* testr32ColData = testr32Col->data();
  real32* testr32Data = testr32ColData->real32Data();
  
  // Populate the columns

  for (unsigned int i  = 0; i < fromTable->rows(); ++i)
    {
      testi16Data[i] = i*10;
      testr32Data[i] = i*100;

    }

  
  // Now create a new dataset

    DataSet* toSet = dataSetServer->open("testCopyTable2.fits", DataSet::Create);

  // Copy the table from the first dataset to the new one

  const Table* newtable = CopyUtils::copyTableToDataSet(fromTable, toSet);

  // Copy the table from the first dataset to the new one, with a new name

  const Table* copy_newtable = CopyUtils::copyTableToDataSet(fromTable, toSet, "A_COPY");

   // Do some checks

  assert(fromTable->rows() == newtable->rows());
  assert(fromTable->rows() == copy_newtable->rows());

  assert(fromTable->columns() == newtable->columns());
  assert(fromTable->columns() == copy_newtable->columns());

  // Cleanup

  dataSetServer->close(toSet);

  dataSetServer->close(fromSet);


  RETURN 0;
} 
